/****************************************************************************
 ****************************************************************************/

#include "jwm.h"

static Cursor defaultCursor;
static Cursor moveCursor;
static Cursor northCursor;
static Cursor southCursor;
static Cursor eastCursor;
static Cursor westCursor;
static Cursor northEastCursor;
static Cursor northWestCursor;
static Cursor southEastCursor;
static Cursor southWestCursor;

static Cursor GetResizeCursor(BorderActionType action);
static Cursor CreateCursor(unsigned int shape);

/****************************************************************************
 ****************************************************************************/
void InitializeCursors() {

	defaultCursor = CreateCursor(XC_left_ptr);
	moveCursor = CreateCursor(XC_fleur);
	northCursor = CreateCursor(XC_top_side);
	southCursor = CreateCursor(XC_bottom_side);
	eastCursor = CreateCursor(XC_right_side);
	westCursor = CreateCursor(XC_left_side);
	northEastCursor = CreateCursor(XC_ur_angle);
	northWestCursor = CreateCursor(XC_ul_angle);
	southEastCursor = CreateCursor(XC_lr_angle);
	southWestCursor = CreateCursor(XC_ll_angle);

}

/****************************************************************************
 ****************************************************************************/
Cursor CreateCursor(unsigned int shape) {
	Cursor cur;
	cur = XCreateFontCursor(display, shape);
	XRecolorCursor(display, cur, &mouseForegroundColor,
		&mouseBackgroundColor);
	return cur;
}

/****************************************************************************
 ****************************************************************************/
void ReleaseCursors() {
	XFreeCursor(display, defaultCursor);
	XFreeCursor(display, moveCursor);
	XFreeCursor(display, northCursor);
	XFreeCursor(display, southCursor);
	XFreeCursor(display, eastCursor);
	XFreeCursor(display, westCursor);
	XFreeCursor(display, northEastCursor);
	XFreeCursor(display, northWestCursor);
	XFreeCursor(display, southEastCursor);
	XFreeCursor(display, southWestCursor);
}

/****************************************************************************
 ****************************************************************************/
int GrabMouseForMove() {
	int result;

	result = XGrabPointer(display, rootWindow, False, ButtonPressMask
		| ButtonReleaseMask | PointerMotionMask, GrabModeAsync,
		GrabModeAsync, None, moveCursor, CurrentTime);

	if(result == GrabSuccess) {
		return 1;
	} else {
		return 0;
	}

}

/****************************************************************************
 ****************************************************************************/
Cursor GetFrameCursor(BorderActionType action) {
	switch(action & 0x0F) {
	case BA_RESIZE:
		return GetResizeCursor(action);
	case BA_CLOSE:
		break;
	case BA_MAXIMIZE:
		break;
	case BA_MINIMIZE:
		break;
	case BA_MOVE:
	default:
		break;
	}
	return defaultCursor;
}

/****************************************************************************
 ****************************************************************************/
Cursor GetResizeCursor(BorderActionType action) {
	if(action & BA_RESIZE_N) {
		if(action & BA_RESIZE_E) {
			return northEastCursor;
		} else if(action & BA_RESIZE_W) {
			return northWestCursor;
		} else {
			return northCursor;
		}
	} else if(action & BA_RESIZE_S) {
		if(action & BA_RESIZE_E) {
			return southEastCursor;
		} else if(action & BA_RESIZE_W) {
			return southWestCursor;
		} else {
			return southCursor;
		}
	} else {
		if(action & BA_RESIZE_E) {
			return eastCursor;
		} else {
			return westCursor;
		}
	}
}

/****************************************************************************
 ****************************************************************************/
int GrabMouseForResize(BorderActionType action) {
	Cursor cur;
	int result;

	cur = GetFrameCursor(action);

	result = XGrabPointer(display, rootWindow, False, ButtonPressMask
		| ButtonReleaseMask | PointerMotionMask, GrabModeAsync,
		GrabModeAsync, None, cur, CurrentTime);

	if(result == GrabSuccess) {
		return 1;
	} else {
		return 0;
	}

}

/****************************************************************************
 ****************************************************************************/
int GrabMouseForMenu() {
	int result;

	result = XGrabPointer(display, rootWindow, False, ButtonPressMask
		| ButtonReleaseMask | PointerMotionMask, GrabModeAsync,
		GrabModeAsync, None, defaultCursor, CurrentTime);

	if(result == GrabSuccess) {
		return 1;
	} else {
		return 0;
	}
		
}

/****************************************************************************
 ****************************************************************************/
void SetDefaultCursor(Window w) {
	XDefineCursor(display, w, defaultCursor);
}

/****************************************************************************
 ****************************************************************************/
void GetMousePosition(int *x, int *y) {
	Window win1, win2;
	int winx, winy;
	unsigned int mask;

	XQueryPointer(display, rootWindow, &win1, &win2, x, y, &winx,
		&winy, &mask);

}

/****************************************************************************
 ****************************************************************************/
unsigned int GetMouseMask() {
	Window win1, win2;
	int rootx, rooty, winx, winy;
	unsigned int mask;

	XQueryPointer(display, rootWindow, &win1, &win2, &rootx, &rooty,
		&winx, &winy, &mask);

	return mask;
}


