
#include "jwm.h"

void FatalError(const char *str, ...) {
	va_list ap;
	va_start(ap, str);

	fprintf(stderr, "error: ");
	vfprintf(stderr, str, ap);
	fprintf(stderr, "\n");

	va_end(ap);

	exit(1);

}

int ErrorHandler(Display *d, XErrorEvent *e) {
#ifdef DEBUG
	char message[80], code[10], request[80];
#endif

	if(initializing) {
		if(e->request_code == X_ChangeWindowAttributes
			&& e->error_code == BadAccess) {
			FatalError("another window manager is already running.");
		}
	}

#ifdef DEBUG

	snprintf(code, sizeof(code), "%d", e->request_code);
	XGetErrorDatabaseText(display, "XRequest", code, "", request,
		sizeof(request));
	if(!request[0]) {
		snprintf(request, sizeof(request), "[request_code=%d]",
			e->request_code);
	}
	if(XGetErrorText(display, e->error_code, message, sizeof(message))
		!= Success) {
		snprintf(message, sizeof(message), "[error_code=%d]",
			e->error_code);
	}

	Debug("XError: %s: %s", request, message);

#endif

	return 0;
}

