/***************************************************************************
 ***************************************************************************/

#include "jwm.h"

typedef struct MenuItemType {
	char *name;
	char *command;
/*
	struct MenuItemType *submenu;
*/
} MenuItemType;

/*
typedef struct MenuType {
	Window win;
	GC gc;
	int x, y;
	int width, height;
} MenuType;
*/

static const MenuItemType MENU_ITEMS[] = {
	{ "xterm", "xterm -rv" },
	{ "xwsh", "xwsh" },
	{ "Mozilla", "mozilla" },
	{ "Gaim", "gaim" },
	{ " ", NULL },
	{ "Exit", "#exit" },
	{ NULL, NULL }
};

static Window menu;
static GC menuGC;

/*
static void ShowSubmenu(MenuItemType *menu, int x, int y);
*/
static void DrawMenu();
static void UpdateMenu(int current, int last);
static void RunCommand(const char *command);

static int menuWidth, menuHeight;
static int menuItemHeight;

/***************************************************************************
 ***************************************************************************/
void ShowRootMenu(int x, int y) {
	XEvent event;
	int selx, sely;
	int lastIndex, currentIndex;
	unsigned int i, temp;

	menuWidth = 4;
	menuHeight = 0;
	menuItemHeight = borderFont->ascent + borderFont->descent + 1;
	for(i = 0; MENU_ITEMS[i].name; i++) {
		menuHeight += menuItemHeight;
		temp = XTextWidth(borderFont, MENU_ITEMS[i].name,
			strlen(MENU_ITEMS[i].name));
		if(temp > menuWidth) {
			menuWidth = temp;
		}
	}
	menuWidth += 8;

	if(x + menuWidth > rootWidth) {
		x = rootWidth - menuWidth;
	}
	if(y + menuHeight > rootHeight - TRAY_HEIGHT - 2) {
		y = rootHeight - TRAY_HEIGHT - menuHeight - 2;
	}

	menu = XCreateSimpleWindow(display, rootWindow, x, y,
		menuWidth, menuHeight, 1, borderOutlineColor.pixel,
		borderColor.pixel);
	XMapRaised(display, menu);
	XSetInputFocus(display, menu, RevertToPointerRoot, CurrentTime);
	menuGC = XCreateGC(display, menu, 0, NULL);
	XSetFont(display, menuGC, borderFont->fid);

	DrawMenu();

	if(!GrabMouseForMenu()) {
		XFreeGC(display, menuGC);
		XDestroyWindow(display, menu);
		return;
	}

	lastIndex = -1;
	for(;;) {

		XMaskEvent(display, ButtonPressMask | PointerMotionMask, &event);

		switch(event.type) {
		case MotionNotify:
			selx = event.xmotion.x - x;
			sely = event.xmotion.y - y;

			if(selx > 0 && sely > 0 && selx < menuWidth && sely < menuHeight) {
				currentIndex = sely / menuItemHeight;
			} else {
				currentIndex = -1;
			}
			if(lastIndex != currentIndex) {
				UpdateMenu(currentIndex, lastIndex);
				lastIndex = currentIndex;
			}

			break;
		case ButtonPress:
			selx = event.xmotion.x - x;
			sely = event.xmotion.y - y;
			goto SelectionMade;
		default:
			break;
		}
	}

SelectionMade:
	XUngrabPointer(display, CurrentTime);

	if(selx > 0 && sely > 0 && selx < menuWidth && sely < menuHeight) {
		RunCommand(MENU_ITEMS[sely / menuItemHeight].command);
	}

	XFreeGC(display, menuGC);
	XDestroyWindow(display, menu);

	RefocusClient();

}

/***************************************************************************
 ***************************************************************************/
/*
void ShowSubmenu(MenuItemType *menu, int x, int y) {
	XEvent event;
	Window menu;
	int selx, sely;
	int lastIndex, currentIndex;
	unsigned int i, temp;

	menuWidth = 4;
	menuHeight = 0;
	menuItemHeight = borderFont->ascent + borderFont->descent + 1;
	for(i = 0; MENU_ITEMS[i].name; i++) {
		menuHeight += menuItemHeight;
		temp = XTextWidth(borderFont, MENU_ITEMS[i].name,
			strlen(MENU_ITEMS[i].name));
		if(temp > menuWidth) {
			menuWidth = temp;
		}
	}
	menuWidth += 8;

	if(x + menuWidth > rootWidth) {
		x = rootWidth - menuWidth;
	}
	if(y + menuHeight > rootHeight - TRAY_HEIGHT - 2) {
		y = rootHeight - TRAY_HEIGHT - menuHeight - 2;
	}

	menu = XCreateSimpleWindow(display, rootWindow, x, y,
		menuWidth, menuHeight, 1, borderOutlineColor.pixel,
		borderColor.pixel);
	XMapRaised(display, menu);
	XSetInputFocus(display, menu, RevertToPointerRoot, CurrentTime);
	menuGC = XCreateGC(display, menu, 0, NULL);
	XSetFont(display, menuGC, borderFont->fid);

	DrawMenu(menu);

	if(!GrabMouseForMenu()) {
		XFreeGC(display, menuGC);
		XDestroyWindow(display, menu);
		return;
	}

	lastIndex = -1;
}
*/

/***************************************************************************
 ***************************************************************************/
void DrawMenu() {
	unsigned int x;

	XSetForeground(display, menuGC, borderColor.pixel);
	XFillRectangle(display, menu, menuGC, 0, 0, menuWidth, menuHeight);
	XSetForeground(display, menuGC, borderTextColor.pixel);
	XSetBackground(display, menuGC, borderColor.pixel);

	for(x = 0; MENU_ITEMS[x].name; x++) {
		XDrawImageString(display, menu, menuGC, 4,
			borderFont->ascent + x * menuItemHeight,
			MENU_ITEMS[x].name, strlen(MENU_ITEMS[x].name));
	}

}

/***************************************************************************
 ***************************************************************************/
void UpdateMenu(int current, int last) {

	/* Clear the old selection. */
	if(last >= 0 && MENU_ITEMS[last].name) {
		XSetForeground(display, menuGC, borderColor.pixel);
		XFillRectangle(display, menu, menuGC, 0, last * menuItemHeight,
			menuWidth, menuItemHeight + 2);
		XSetForeground(display, menuGC, borderTextColor.pixel);
		XSetBackground(display, menuGC, borderColor.pixel);
		XDrawImageString(display, menu, menuGC, 4,
			borderFont->ascent + last * menuItemHeight,
			MENU_ITEMS[last].name, strlen(MENU_ITEMS[last].name));
	}

	/* Highlight the new selection. */
	if(current >= 0 && MENU_ITEMS[current].name) {
		XSetForeground(display, menuGC, activeBorderColor.pixel);
		XFillRectangle(display, menu, menuGC, 0, current * menuItemHeight,
			menuWidth, menuItemHeight);
		XSetForeground(display, menuGC, activeBorderTextColor.pixel);
		XSetBackground(display, menuGC, activeBorderColor.pixel);
		XDrawImageString(display, menu, menuGC, 4,
			borderFont->ascent + current * menuItemHeight,
			MENU_ITEMS[current].name, strlen(MENU_ITEMS[current].name));
	}

}

/***************************************************************************
 ***************************************************************************/
void RunCommand(const char *command) {
	char *displayString;
	char *str;

	if(!command) {
		return;
	}

	if(!strcmp(command, "#exit")) {
		shouldExit = 1;
		return;
	}

	displayString = DisplayString(display);

	if(!fork()) {
		if(!fork()) {
			close(ConnectionNumber(display));

			if(displayString && displayString[0]) {
				str = malloc(strlen(displayString) + 9);
				sprintf(str, "DISPLAY=%s", displayString);
				putenv(str);
			}

			execl(SHELL_NAME, SHELL_NAME, "-c", command, NULL);
			fprintf(stderr, "exec failed\n");
			exit(1);

		}
		exit(0);
	}

	wait(NULL);

}

